package eu.mhsl.craftattack.spawn.appliances.feedback;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.feedback.commands.FeedbackCommand;
import eu.mhsl.craftattack.spawn.appliances.feedback.commands.RequestFeedbackCommand;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.*;

public class Feedback extends Appliance {
    private final URI apiEndpoint;

    public Feedback() {
        super("feedback");
        this.apiEndpoint = URI.create(Objects.requireNonNull(localConfig().getString("api")));
    }

    public void requestFeedback(String eventName, List<Player> receivers, @Nullable String question) {
        Map<UUID, String> feedbackUrls = createPersonalizedUrls(
            new Request(eventName, receivers.stream().map(Entity::getUniqueId).toList())
        );

        System.out.println(feedbackUrls.toString());

        Component border = Component.text("-".repeat(40), NamedTextColor.GRAY);

        receivers.forEach(player -> {
            String feedbackUrl = feedbackUrls.get(player.getUniqueId());
            if(feedbackUrl == null) {
                Main.logger().warning(String.format("FeedbackUrl not found for player '%s' from backend!", player.getUniqueId()));
                return;
            }

            ComponentBuilder<TextComponent, TextComponent.Builder> message = Component.text()
                .append(border)
                .appendNewline();

            if(question != null) {
                message
                    .append(Component.text(question, NamedTextColor.GREEN))
                    .appendNewline()
                    .appendNewline();
            }

            message
                .append(Component.text("Klicke hier und gib uns Feedback, damit wir dein Spielerlebnis verbessern können!", NamedTextColor.DARK_GREEN)
                    .clickEvent(ClickEvent.openUrl(feedbackUrl)))
                .hoverEvent(HoverEvent.showText(Component.text("Klicke, um Feedback zu geben.")))
                .appendNewline()
                .append(border);

            player.sendMessage(message.build());
        });
    }

    private record Request(String event, List<UUID> users) {}
    private final Type responseType = new TypeToken<Map<UUID, String>>(){}.getType();
    private Map<UUID, String> createPersonalizedUrls(Request data) {
        try(HttpClient client = HttpClient.newHttpClient()) {
            HttpRequest httpRequest = HttpRequest.newBuilder()
                .uri(this.apiEndpoint)
                .header("Content-Type", "application/json")
                .POST(HttpRequest.BodyPublishers.ofString(new Gson().toJson(data)))
                .build();

            HttpResponse<String> httpResponse = client.send(httpRequest, HttpResponse.BodyHandlers.ofString());
            return new Gson().fromJson(httpResponse.body(), responseType);
        } catch(IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(
            new FeedbackCommand(),
            new RequestFeedbackCommand()
        );
    }
}
